#!/usr/bin/env python
#
# This file is part of A Musical Adventure
# Author(s): Kelley Stidolph, Girlstart
# Copyright 2008
#
# Source code is derived from Girlstart's Project IT Girl Game Templates:
# http://itgirl.dreamhosters.com/itgirlgames/, http://www.girlstart.org/itgirl
# See the credits file for image sources.
#
# A Musical Adventure is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# A Musical Adventure is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with A Musical Adventure.  If not, see <http://www.gnu.org/licenses/>.

import string, pygame
from Game import *
from Room import *
from helpers import *

# Draw a colored Rectangle with transparent center - no drawing there, just the outside lines
def DrawPicRectOutline(surface, rc, color, thickness):
	surface.fill(color,Rect(rc.left,rc.top,rc.width,thickness))
	surface.fill(color,Rect(rc.right-thickness,rc.top,thickness,rc.height))
	surface.fill(color,Rect(rc.left,rc.bottom-thickness,rc.width,thickness))
	surface.fill(color,Rect(rc.left,rc.top,thickness,rc.height))
	
class PicSquare:

	def __init__(self, rc, wave):
		self.rc = rc
		self.selected = False
		self.solved = False
		self.Sound = wave
		
	def Draw(self, surface):
		if not self.solved:
			if self.selected:
				color = (192,192,192)
			else:
				color = (128,128,128)
			surface.fill(color,self.rc)
			DrawPicRectOutline(surface, self.rc, (0,0,0), 2)
			
class Concentration(Game):
			
	def __init__(self):
		Game.__init__(self)
		self.leftSquares = []
		self.rightSquares = []
		self.timerState = 0
		
	def Initialize(self, screen, textArea):
		Game.Initialize(self, screen, textArea)
		self.sounds = []
		for i in range(21,33):
			name = "Piano_"+str(i)+".wav"
			self.sounds.append(load_sound(name))
		totalWidth = self.picLeftRect.width + self.picRightRect.width + 10
		left1 = textArea.centerx - (totalWidth/2)
		left2 = left1 + self.picLeftRect.width + 10
		top = self.NextLineY
		self.picLeftRect.move_ip(left1,top)
		self.picRightRect.move_ip(left2,top)
		w1 = self.picLeftRect.width / 3
		w2 = self.picRightRect.width / 3
		h1 = self.picLeftRect.height / 4
		h2 = self.picRightRect.height / 4
		y1 = top
		y2 = top
		i = 0
		for row in range(0,4):
			x1 = left1
			x2 = left2
			for col in range(0,3):
				self.leftSquares.append(PicSquare(Rect(x1,y1,w1,h1),self.sounds[i]))
				self.rightSquares.append(PicSquare(Rect(x2,y2,w2,h2),self.sounds[i]))
				x1 += w1
				x2 += w2
				i += 1
			y1 += h1
			y2 += h2
		self.leftSquares[0].Sound = self.sounds[5]
		self.leftSquares[1].Sound = self.sounds[8]
		self.leftSquares[2].Sound = self.sounds[3]
		self.leftSquares[3].Sound = self.sounds[11]
		self.leftSquares[4].Sound = self.sounds[0]
		self.leftSquares[5].Sound = self.sounds[4]
		self.leftSquares[6].Sound = self.sounds[9]
		self.leftSquares[7].Sound = self.sounds[10]
		self.leftSquares[8].Sound = self.sounds[1]
		self.leftSquares[9].Sound = self.sounds[7]
		self.leftSquares[10].Sound = self.sounds[2]
		self.leftSquares[11].Sound = self.sounds[6]
		
		self.rightSquares[0].Sound = self.sounds[11]
		self.rightSquares[1].Sound = self.sounds[1]
		self.rightSquares[2].Sound = self.sounds[3]
		self.rightSquares[3].Sound = self.sounds[9]
		self.rightSquares[4].Sound = self.sounds[6]
		self.rightSquares[5].Sound = self.sounds[2]
		self.rightSquares[6].Sound = self.sounds[10]
		self.rightSquares[7].Sound = self.sounds[4]
		self.rightSquares[8].Sound = self.sounds[7]
		self.rightSquares[9].Sound = self.sounds[5]
		self.rightSquares[10].Sound = self.sounds[8]
		self.rightSquares[11].Sound = self.sounds[0]

	def Dump(self):
		print "--- Game ---"
		print "Name: " + self.Name
			
	def Load(self, attrs):
		Game.Load(self, attrs)
		self.picLeft, self.picLeftRect = load_image("concentration1.jpg")
		self.picRight, self.picRightRect = load_image("concentration2.jpg")
		for attrName in attrs.keys():
			value = attrs.get(attrName)
				
	def Draw(self, surface, textArea):
		# Do the drawing
		Game.Draw(self, surface, textArea)
		surface.blit(self.picLeft, self.picLeftRect)
		surface.blit(self.picRight, self.picRightRect)
		for pic in self.leftSquares:
			pic.Draw(surface)
		for pic in self.rightSquares:
			pic.Draw(surface)
		DrawPicRectOutline(surface, self.picLeftRect, (0,0,0), 2)
		DrawPicRectOutline(surface, self.picRightRect, (0,0,0), 2)
	
	def OnTimer(self):
		if self.timerState>1:
			self.timerState = self.timerState-1
			return False
		elif self.timerState == 1:
			left = self.GetSelectedLeft()
			right = self.GetSelectedRight()
			
			self.timerState = 0
			return True
			
	# Called for a Key Press
	# return True if redraw required or room change
	# return False if no change
	def OnKeyDown(self, unicode, key, mod):
		#Key has been pressed
		if key == 283: #F2
			self.NextRoom = self.OnSuccess
			return True
		return Game.OnKeyDown(self, unicode,key, mod)
	
	# Called for a Key Release
	# return True if redraw required or room change
	# return False if no change
	def OnKeyUp(self, key, mod):
		#Key has been release
		print "Key Up: %d" % (key)
		return False
	
	def GetSelectedLeft(self):
		for square in self.leftSquares:
			if square.selected:
				return square
		return None
		
	def GetSelectedRight(self):
		for square in self.rightSquares:
			if square.selected:
				return square
		return None
		
	def GetSquare(self, pos):
		for square in self.leftSquares:
			if square.rc.collidepoint(pos[0], pos[1]):
				return square
		for square in self.rightSquares:
			if square.rc.collidepoint(pos[0], pos[1]):
				return square
		return None
		
	# Called when the mouse moves
	# return True if redraw required or room change
	# return False if no change
	def OnMouseMove(self, pos, rel, buttons):
		#Mouse has been moved
		square = self.GetSquare(pos)
		return False
	
	# Called for a mouse button pressed
	# return True if redraw required or room change
	# return False if no change
	def OnMouseButtonDown(self, pos, button):
		square = self.GetSquare(pos)
		if square <> None:
			if not square.solved:
				if self.picLeftRect.collidepoint(pos[0], pos[1]):
					for sq in self.leftSquares:
						if not sq.solved and sq.selected and (sq <> square):
							sq.selected = False
				elif self.picRightRect.collidepoint(pos[0], pos[1]):
					for sq in self.rightSquares:
						if not sq.solved and sq.selected and (sq <> square):
							sq.selected = False
 				square.selected = not square.selected
				if square.selected:
					square.Sound.play()
				selectedLeft = self.GetSelectedLeft()
				selectedRight = self.GetSelectedRight()
				if selectedLeft and selectedRight:
					if selectedLeft.Sound == selectedRight.Sound:
						selectedLeft.solved = True
						selectedRight.solved = True
						selectedLeft.selected = False
						selectedRight.selected = False
						allDone = True
						for sq in self.leftSquares:
							if not sq.solved:
								allDone = False
						for sq in self.rightSquares:
							if not sq.solved:
								allDone = False
						if allDone:
							self.NextRoom = self.OnSuccess
				return True
		return False
		
	# Called for a mouse button release
	# return True if redraw required or room change
	# return False if no change
	def OnMouseButtonUp(self, pos, button):
		#Mouse button up
		print 'OnMouseButtonUp (%d,%d)' % pos
		return False
		
	def GetNextRoom(self):
		return self.NextRoom

	def OnEnter(self, lastRoom):
		Game.OnEnter(self, lastRoom)
		self.NextRoom = self.Name
		
	def OnExit(self, nextRoom):
		Game.OnExit(self, nextRoom)
	
	
