#!/usr/bin/env python
#
# This file is part of A Musical Adventure
# Author(s): Kelley Stidolph, Girlstart
# Copyright 2008
#
# Source code is derived from Girlstart's Project IT Girl Game Templates:
# http://itgirl.dreamhosters.com/itgirlgames/, http://www.girlstart.org/itgirl
# See the credits file for image sources.
#
# A Musical Adventure is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# A Musical Adventure is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with A Musical Adventure.  If not, see <http://www.gnu.org/licenses/>.

import sys, pygame
from Room import *
from Game import *
from helpers import *

class Keypad:
	def __init__(self,x,y,width,height,sound,isCode,keycode):
		self.x = x
		self.y = y
		self.left = x
		self.right = x+width-1
		self.top = y
		self.bottom = y+height-1
		self.sound = sound
		self.isCode = isCode
		self.keycode = keycode
		self.srcRect = Rect(x,y,width,height)
		self.destRect = None
		self.selected = False
		
	def InitDest(self, left, top):
		self.destRect = Rect(left+self.x, top+self.y, self.srcRect.width, self.srcRect.height)
		
	def PtInPad(self, x, y):
		if x>=self.left and x<=self.right and y>=self.top and y<=self.bottom:
			return True
		else:
			return False
		
class KeypadGame(Game):
	def __init__(self):
		Game.__init__(self)
		self.listenButtonDest = None
		self.queuedSounds = []
		self.digitReadoutDest = None
		self.Channel = None
		self.keypadRect = None
		self.clearButtonRect = None
		
	def Initialize(self, screen, textArea):
		Game.Initialize(self, screen, textArea)
		self.code = [1,2,3,4]
		self.typedCode = []
		self.keypads = []
		self.keypads.append(Keypad(56,91,19,19,self.sounds[0],False,0))
		self.keypads.append(Keypad(30,35,18,16,self.sounds[1],False,1))
		self.keypads.append(Keypad(56,34,17,17,self.sounds[2],True,2))
		self.keypads.append(Keypad(81,34,17,17,self.sounds[3],False,3))
		self.keypads.append(Keypad(30,53,18,17,self.sounds[4],False,4))
		self.keypads.append(Keypad(56,52,18,18,self.sounds[5],True,5))
		self.keypads.append(Keypad(81,52,19,18,self.sounds[6],False,6))
		self.keypads.append(Keypad(30,72,19,19,self.sounds[7],False,7))
		self.keypads.append(Keypad(56,72,19,18,self.sounds[8],True,8))
		self.keypads.append(Keypad(82,71,19,18,self.sounds[9],False,9))
		self.left = screen.get_width()/2 - self.keypadGraphicRect.width/2
		self.listenButton, self.listenButtonRect = load_image("listen.jpg")
		self.top = self.NextLineY + self.listenButtonRect.height + 5
		
	def Load(self, attrs):
		Game.Load(self, attrs)
		self.digitDisplay, self.digitDisplayRect = load_image("DigitDisplay.jpg")
		self.digitsGraphic, self.digitsGraphicRect = load_image("digits.jpg")
		self.digitRect = []
		self.digitWidth = self.digitsGraphicRect.width / 10
		for i in range(0,10):
			self.digitRect.append(Rect(i*self.digitWidth,0,self.digitWidth,self.digitsGraphicRect.height))
		self.keypadGraphic, self.keypadGraphicRect = load_image("keypad1.jpg")
		self.keypadHighlightGraphic, self.keypadHighlightGraphicRect = load_image("keypad1.jpg")
		for attrName in attrs.keys():
			value = attrs.get(attrName)
		self.sounds = []
		for i in range(21,31):
			name = "Piano_"+str(i)+".wav"
			self.sounds.append(load_sound(name))
		
	def Draw(self, surface, textArea):
		# Do the drawing
		Game.Draw(self, surface, textArea)
		playY = self.NextLineY
		if self.listenButtonDest == None:
			x = textArea.centerx - self.listenButtonRect.width/2
			self.listenButtonDest = Rect(x, playY+2, self.listenButtonRect.width, self.listenButtonRect.height)
			playY = playY + self.listenButtonRect.height + 5
		surface.blit(self.listenButton, self.listenButtonDest)
		if self.digitReadoutDest == None:
			x = textArea.centerx - self.digitDisplayRect.width/2
			y = playY+2
			self.digitReadoutDest = Rect(x, y, self.digitDisplayRect.width, self.digitDisplayRect.height)
			playY = playY + self.digitDisplayRect.height + 5
		surface.blit(self.digitDisplay, self.digitReadoutDest)
		y = self.digitReadoutDest.top+1
		x = self.digitReadoutDest.left+1
		for digit in self.typedCode:
			destRect = Rect(x,y,self.digitWidth, self.digitsGraphicRect.height)
			srcRect = Rect(digit*self.digitWidth,0,self.digitWidth, self.digitsGraphicRect.height)
			surface.blit(self.digitsGraphic,destRect,srcRect)
			x = x + self.digitWidth
		if self.keypadRect == None:
			x = textArea.centerx - self.keypadGraphicRect.width/2
			y = playY+2
			self.keypadRect = Rect(x, y, self.keypadGraphicRect.width, self.keypadGraphicRect.height)
			for pad in self.keypads:
				pad.InitDest(x,y)
			self.clearButtonRect = Rect(x+38, y+127, 56, 16)
		surface.blit(self.keypadGraphic, self.keypadRect)
		playY = self.keypadRect.bottom
	
	def OnTimer(self):
		if len(self.queuedSounds) > 0:
			if not pygame.mixer.get_busy():
				sound = self.queuedSounds[0]
				self.queuedSounds = self.queuedSounds[1:]
				sound.play()
		if not pygame.mixer.get_busy():
			if len(self.code) == len(self.typedCode):
				for i in range(0,len(self.code)):
					if self.code[i] <> self.typedCode[i]:
						return False
				self.NextRoom = self.OnSuccess
				return True
		return False
		
	# Called for a Key Press
	# return True if redraw required or room change
	# return False if no change
	def OnKeyDown(self, unicode, key, mod):
		#Key has been pressed
		return Room.OnKeyDown(self, unicode, key, mod)
	
	# Called for a Key Release
	# return True if redraw required or room change
	# return False if no change
	def OnKeyUp(self, key, mod):
		#Key has been release
		return Game.OnKeyUp(self, key, mod)
		
	# Called when the mouse moves
	# return True if redraw required or room change
	# return False if no change
	def OnMouseMove(self, pos, rel, buttons):
		#Mouse has been moved
		return False
	
	# Called for a mouse button pressed
	# return True if redraw required or room change
	# return False if no change
	def OnMouseButtonDown(self, pos, button):
		if self.listenButtonDest.collidepoint(pos):
			print "Listen Button!"
			channel = pygame.mixer.Channel(0)
			for code in self.code:
				self.queuedSounds.append(self.keypads[code].sound)
			return False
		for pad in self.keypads:
			if pad.destRect.collidepoint(pos):
				print "In pad"
				if len(self.typedCode) < 4:
					self.typedCode.append(pad.keycode)
					self.queuedSounds.append(pad.sound)				
				return True
		if self.clearButtonRect.collidepoint(pos):
			print "Clear Button!"
			self.typedCode = []
			return True
		return False
		
	# Called for a mouse button release
	# return True if redraw required or room change
	# return False if no change
	def OnMouseButtonUp(self, pos, button):
		#Mouse button up
		return False
		
	def GetNextRoom(self):
		return self.NextRoom

	def OnEnter(self, lastRoomName):
		self.codeTyped = []
		self.NextRoom = self.Name
		print "OnEnter: " + self.Name
		Game.OnEnter(self, lastRoomName)
		
	def OnExit(self, nextRoomName):
		Game.OnExit(self, nextRoomName)
		print "OnExit: " + self.Name	
	
	
