#!/usr/bin/env python
#
# This file is part of A Musical Adventure
# Author(s): Kelley Stidolph, Girlstart
# Copyright 2008
#
# Source code is derived from Girlstart's Project IT Girl Game Templates:
# http://itgirl.dreamhosters.com/itgirlgames/, http://www.girlstart.org/itgirl
# See the credits file for image sources.
#
# A Musical Adventure is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# A Musical Adventure is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with A Musical Adventure.  If not, see <http://www.gnu.org/licenses/>.

import sys, pygame, random, os, string
from helpers import *
from Activities import *
from xml.sax import handler, make_parser
from Option import *
from Room import *
from Game import *
from Concentration import *
from KeypadGame import *

class MyXMLDocumentHandler(handler.ContentHandler):
	def __init__(self, roomList):
		self.room = None
		self.game = None
		self.roomList = roomList
		
	def startElement(self, name, attrs):
		if name == 'Room':
			self.room = Room()
			self.room.Load(attrs)
		elif name == 'Option':
			option = Option()
			option.Load(attrs)
			self.room.AddOption(option)
		elif name == 'Game':
			if attrs['Name']=='Concentration':
				self.game = Concentration()
				self.game.Load(attrs)
			elif attrs['Name']=='KeypadGame':
				self.game = KeypadGame()
				self.game.Load(attrs)
			
	def endElement(self, name):
		if name == 'Room':
			if self.room <> None:
				self.roomList.append(self.room)
				self.room = None
		elif name == 'Game':
			if self.game <> None:
				self.roomList.append(self.game)
				self.game = None

def GetRoomFromName(roomList, roomName):
	for room in roomList:
		if room.Name == roomName:
			return room
	print "Could not find room: " + roomName
	return None

def ProcessEvent(event, currentRoom):
	somethingChanged = False
	if event.type == pygame.QUIT:
		sys.exit()
	elif event.type == pygame.KEYDOWN:
		unicode = event.unicode
		key = event.key
		mod = event.mod
		somethingChanged = currentRoom.OnKeyDown(unicode, key, mod)
	elif event.type == pygame.KEYUP:
		key = event.key
		mod = event.mod
		somethingChanged = currentRoom.OnKeyUp(key, mod)
	elif event.type == pygame.MOUSEMOTION:
		pos = event.pos
		rel = event.rel
		buttons = event.buttons
		somethingChanged = currentRoom.OnMouseMove(pos, rel, buttons)
	elif event.type == pygame.MOUSEBUTTONUP:
		pos = event.pos
		button = event.pos
		somethingChanged = currentRoom.OnMouseButtonUp(pos, button)
	elif event.type == pygame.MOUSEBUTTONDOWN:
		pos = event.pos
		button = event.pos
		somethingChanged = currentRoom.OnMouseButtonDown(pos, button)
	elif event.type == (pygame.USEREVENT+1):
		somethingChanged = currentRoom.OnTimer()
	return somethingChanged

def Draw(surface, currentRoom, roomArea):
	surface.blit(background,(0,0))
	currentRoom.Draw(surface, roomArea)
	pygame.display.flip()
	
pygame.init()

# Set up the window
windowsize = windowwidth, windowheight = 800, 600
surface = pygame.display.set_mode(windowsize)
area = surface.get_rect()
background,background_rect = load_image("background.png")

# Read Rooms.xml and process into game data
rooms = []
handler = MyXMLDocumentHandler(rooms)
parser = make_parser()
parser.setContentHandler(handler)
inFile = open('Rooms.xml', 'r')
parser.parse(inFile)
inFile.close()

roomArea = area
roomArea.top += 42 #below line at top

# Print out Rooms to verify data
for room in rooms:
	room.Initialize(surface, roomArea)
	room.Dump()

currentRoom = rooms[0];
currentRoom.OnEnter("")

# Main game play
redrawRequired = True #Force draw the first time
pygame.time.set_timer(pygame.USEREVENT+1, 100)
lastRoomName = ""
while True:
	# Check to see if the user has clicked the mouse or pressed a key
	for event in pygame.event.get():
		changed = ProcessEvent(event, currentRoom)
		if changed:
			redrawRequired = True
		nextRoomName = currentRoom.GetNextRoom()
		if nextRoomName == "":
			print "The Room: " + currentRoom.Name + " is returning an empty string for the next room"
			sys.exit()
		if nextRoomName != currentRoom.Name:
			currentRoom.OnExit(nextRoomName)
			if nextRoomName == "Pop":
				nextRoomName = lastRoomName
			lastRoomName = currentRoom.Name
			currentRoom = GetRoomFromName(rooms, nextRoomName)
			if currentRoom == None:
				print "Looking for Room: " + nextRoomName + " could not find that room in xml"
				sys.exit()
			currentRoom.OnEnter(lastRoomName)
	if redrawRequired:
		Draw(surface, currentRoom, roomArea)
		redrawRequired = False
