#!/usr/bin/env python
#
# This file is part of A Musical Adventure
# Author(s): Kelley Stidolph, Girlstart
# Copyright 2008
#
# Source code is derived from Girlstart's Project IT Girl Game Templates:
# http://itgirl.dreamhosters.com/itgirlgames/, http://www.girlstart.org/itgirl
# See the credits file for image sources.
#
# A Musical Adventure is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# A Musical Adventure is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with A Musical Adventure.  If not, see <http://www.gnu.org/licenses/>.

import string, pygame
from helpers import *

class Room:
	def __init__(self):
		self.font = pygame.font.Font(None, 30)
		self.Clear()
		
	def Clear(self):
		self.Name = "" #Name of the room
		self.NextRoom = ""
		self.Cmd = ""
		self.lastRoom = ""
		self.Background = None
		self.BackgroundRect = None
		self.Graphic = None
		self.GraphicRect = None
		self.NextLineY = 0
		self.textBoxRect = pygame.Rect(0,20,800,580)
		self.textBoxSet = False
		self.fontColor = (0,0,0)
		self.fontName = "Arial"
		self.Text = [] #Ordered list of text lines for room
		self.TextArea = [] #Where to draw the text in Rect
		self.Options = [] #Ordered list of selections/options/choices for room
		self.textData = {} #create an empty dictionary to hold the text data (temporary for loading)
		
	def Load(self, attrs):
		textLeft = 0;
		textRight = 800;
		textTop = 0;
		textBottom = 600;
		textSet = False;
		for attrName in attrs.keys():
			value = attrs.get(attrName)
			if attrName=="Name":
				self.Name = value
				self.NextRoom = value
			elif attrName == "Cmd":
				self.Cmd = value
			elif attrName == "TextBoxLeft":
				textLeft = int(value)
				textSet = True
			elif attrName == "TextBoxTop":
				textTop = int(value)
				textSet = True
			elif attrName == "TextBoxRight":
				textRight = int(value)
				textSet = True
			elif attrName == "TextBoxBottom":
				textBottom = int(value)
				textSet = True
			elif attrName[:4] == "Text":
				self.textData[attrName] = value
			elif attrName == "Graphic":
				self.Graphic, self.GraphicRect = load_image(value)
			elif attrName == "GraphicX":
				self.GraphicRect.move_ip(int(value),self.GraphicRect.top)
			elif attrName == "GraphicY":
				self.GraphicRect.move_ip(self.GraphicRect.left,int(value))
			elif attrName == "Background":
				self.Background, self.BackgroundRect = load_image(value)
			elif attrName == "FontColor":
				if value == "black":
					self.fontColor = (0,0,0)
				elif value == "white":
					self.fontColor = (255,255,255)
				elif value == "yellow":
					self.fontColor = (255,255,0)
				elif value == "blue":
					self.fontColor = (0,0,255)
				elif value == "green":
					self.fontColor = (0,255,0)
				elif value == "red":
					self.fontColor = (255,0,0)
				elif value == "grey":
					self.fontColor = (192,192,192)
				else:
					print "Uknown color name" + value
		if textSet:
			self.textBoxRect = Rect(textLeft, textTop, textRight-textLeft, textBottom-textTop)
			print "TextBox (%d,%d,%d,%d)" %(textLeft, textTop, textRight, textBottom)
		self.textBoxSet = textSet
				
		# Order the text lines by name and save in a list
		keys = self.textData.keys()
		keys.sort()
		for key in keys:
			value = self.textData.get(key)
			self.Text.append(value)
			self.TextArea.append(pygame.Rect(0,0,0,0))

	def AddOption(self, Option):
		self.Options.append(Option)
		
	def Dump(self):
		print "--- Room ---"
		print "Name: " + self.Name
		index = 0
		for line in self.Text:
			print "Text: " + line
			rc = self.TextArea[index]
			index += 1
			print 'x=%d, y=%d, w=%d, h=%d\n' % (rc.x, rc.y, rc.width, rc.height)
		for option in self.Options:
			option.Dump()

	def GetTextArea(self, text):
		if text=="":
			text = " "
		surface = self.font.render(text, 1, (0,0,0))
		rc = surface.get_rect()
		print 'Text='+text+' width = %d, height = %d\n' % (rc.width, rc.height)
		return rc
		
	def Initialize(self, screen, textArea):
		# Setup text and option areas for display
		if self.textBoxSet:
			textArea = self.textBoxRect
		self.NextLineY = textArea.top + 2
		maxWidth = 0
		index = 0
		for line in self.Text:
			rc = self.GetTextArea(line)
			rc.move_ip(0,self.NextLineY)
			if rc.width > maxWidth:
				maxWidth = rc.width
			self.TextArea[index] = rc
			self.NextLineY += rc.height
			index += 1
		x = textArea.centerx - (maxWidth/2)
		if x < textArea.left:
			x = textArea.left
		for line in self.TextArea:
			line.move_ip(x,0)
		
		#Now for the options
		self.NextLineY += 20
		maxWidth = 0
		for option in self.Options:
			width = option.Initialize(self.font, self.NextLineY)
			if (width > maxWidth):
				maxWidth = width
			self.NextLineY += option.TextArea.height
		x = textArea.centerx - (maxWidth/2)
		if x < textArea.left:
			x = textArea.left
		for option in self.Options:
			option.TextArea.move_ip(x,0)
				
	def Draw(self, screen, textArea):
		# Draw text and options on screen
		if self.Background:
			screen.blit(self.Background, (0,0))
		if self.Graphic:
			screen.blit(self.Graphic,self.GraphicRect) 
		index = 0;
		for line in self.Text:
			surface = self.font.render(line, 1, self.fontColor)
			screen.blit(surface, self.TextArea[index])
			index += 1
		for option in self.Options:
			option.Draw(screen, self.font)
	
	def OnTimer(self):
		return False
		
	# Called for a Key Press
	# return True if redraw required or room change
	# return False if no change
	def OnKeyDown(self, unicode, key, mod):
		#Key has been pressed
		if key == 282: #F1
			self.NextRoom = "Help"
			return True
		for option in self.Options:
			if option.Key == key:
				self.NextRoom = option.Room
				return True
		return False
	
	# Called for a Key Release
	# return True if redraw required or room change
	# return False if no change
	def OnKeyUp(self, key, mod):
		#Key has been release
		print "Key Up: %d" % (key)
		return False
		
	# Called when the mouse moves
	# return True if redraw required or room change
	# return False if no change
	def OnMouseMove(self, pos, rel, buttons):
		#Mouse has been moved
		return False
	
	# Called for a mouse button pressed
	# return True if redraw required or room change
	# return False if no change
	def OnMouseButtonDown(self, pos, button):
		for option in self.Options:
			if option.TextArea.collidepoint(pos[0], pos[1]):
				self.NextRoom = option.Room
				return True
		return False
		
	# Called for a mouse button release
	# return True if redraw required or room change
	# return False if no change
	def OnMouseButtonUp(self, pos, button):
		#Mouse button up
		print "OnMouseButtonUp"
		return False
		
	def GetNextRoom(self):
		return self.NextRoom

	def OnEnter(self, lastRoom):
		print "OnEnter: " + self.Name
		self.NextRoom = self.Name
		self.lastRoom = lastRoom
		
	def OnExit(self, nextRoom):
		print "OnExit: " + self.Name
		
